#!/usr/bin/env python

from behave import configuration
from behave import __main__

## add my wanted option to parser.
configuration.parser.add_argument('--browser')
configuration.parser.add_argument('--browser-size')
configuration.parser.add_argument('--site')
configuration.parser.add_argument('--debug', action='store_true')
configuration.parser.add_argument('--screenshot', action='store_true')

## command that run behave.
def main():
    return __main__.main()



# -*- coding: utf-8 -*-
import glob
import yaml

from Logger import Logger as log

class Responsive():

    browser_sizes = {}

    @classmethod
    def load_browser_sizes(cls, filename):
        content = open(filename, "r")
        doc = yaml.load(content)

        for item in doc['browser_sizes']:
            try:
                name = item['name']
                size = item['size']

                width = int(size.split('x')[0].strip())
                height = int(size.split('x')[1].strip())

                cls.browser_sizes[name] = {
                    'width': width,
                    'height': height
                }
            except:
                log.failed("Invalid responsive configuration, please check");

    @classmethod
    def get_browser_size(cls, name):
        if name in cls.browser_sizes:
            return cls.browser_sizes[name]
        else:
            log.failed("Not found responsive config '%s'" % name)



# -*- coding: utf-8 -*-
from Logger import *
from Responsive import *

from CommonSteps import *
from PageFactory import *

from World import World as world

class BehaveController:

    @classmethod
    def before_all(cls, context):
        Logger.tracebacklimit(0)
        if context.config.debug == True:
            Logger.tracebacklimit(1000)

        if context.config.screenshot == True:
            Logger.enable_screenshots()

        Responsive.load_browser_sizes("./responsive.yaml")

        site = "default"
        if hasattr(context.config, "site") and context.config.site is not None:
            site = context.config.site
        domain = PageFactory.get_domain("./config.yaml", site)

        pageobjects = "./pageobjects/*.yaml"
        if PageFactory.check_yaml_syntax(pageobjects) == True:
            world.pages = PageFactory.create_pages(pageobjects, domain)
        else:
            Logger.failed("Stop running.")

    @classmethod
    def after_all(cls, context):
        pass

    @classmethod
    def before_feature(cls, context, feature):
        if hasattr(context.config, "browser_size") and context.config.browser_size is not None:
            name = context.config.browser_size
            size = Responsive.get_browser_size(name)
            world.size = size

        browser = "firefox"
        if hasattr(context.config, "browser") and context.config.browser is not None:
            browser = context.config.browser

        world.open_browser(browser)
        Logger.driver = world.driver

    @classmethod
    def after_feature(cls, context, feature):
        world.close_browser()



# -*- coding: utf-8 -*-
import os
import sys
import shutil

class Logger:

    driver = None

    screenshot_path = "./screenshots"
    screenshot_number = 1
    screenshot_enabled = False

    @classmethod
    def enable_screenshots(cls):
        cls.screenshot_enabled = True

        if os.path.exists(cls.screenshot_path):
            shutil.rmtree(cls.screenshot_path)

        os.makedirs(cls.screenshot_path)

    @classmethod
    def tracebacklimit(cls, value=0):
        sys.tracebacklimit = int(value)

    @classmethod
    def warning(cls, message, fields={}):
        message = "WARN: %s\n" % message

        for key, value in fields.iteritems():
            message += "  %s: '%s'\n" % (key, value)

        print message
        raise RuntimeError

    @classmethod
    def failed(cls, message, actual="", expect=""):
        message = "FAIL: %s\n" % message

        if actual != "" and expect != "":
            message += "actual: '%s'\nexpect: '%s'\n" % (actual, expect)

        print message

        if cls.screenshot_enabled == True:
            filename = "%s/%03d.png" % (cls.screenshot_path, cls.screenshot_number)
            cls.driver.save_screenshot(filename)
            cls.screenshot_number += 1

            print "screenshot: '%s'" % filename

        raise RuntimeError
